@@ -15,6 +15,7 @@
 #include <QFileDialog>
 #include <QImageWriter>
 #include <QLocale>
+#include <QLockFile>
 #include <QMimeData>
 #include <QMimeDatabase>
 #include <QPainter>
@@ -177,19 +178,33 @@ QUrl ExportManager::tempVideoUrl()
     return QUrl::fromLocalFile(filepath);
 }
 
+void removeUnlockedDirs(const QStringList &oldDirs)
+{
+    for (const auto &dirStr : oldDirs) {
+        QDir dir(QDir::tempPath() + u'/' + dirStr);
+        QLockFile lockFile(dir.filePath(u"lockfile"_s));
+        if (dir.exists() && lockFile.tryLock()) {
+            lockFile.unlock();
+            dir.removeRecursively();
+        }
+    }
+}
+
 const QTemporaryDir *ExportManager::temporaryDir()
 {
     if (!m_tempDir) {
         // Cleanup Spectacle's temp dirs after startup instead of while quitting.
         const auto filters = QDir::Filter::Dirs | QDir::NoDotAndDotDot | QDir::CaseSensitive | QDir::NoSymLinks;
+        // Get old dirs before the async stuff to avoid race conditions when making the new dir.
         const auto oldDirs = QDir::temp().entryList({u"Spectacle.??????"_s}, filters);
-        QList<QUrl> oldUrls;
-        for (const auto &dir : oldDirs) {
-            oldUrls << QUrl::fromLocalFile(QDir::tempPath() + u'/' + dir);
-        }
-        KIO::del(oldUrls, KIO::HideProgressInfo)->start();
         m_tempDir = std::make_unique<QTemporaryDir>(QDir::tempPath() + u"/Spectacle.XXXXXX"_s);
         m_tempDir->setAutoRemove(false);
+        if (m_tempDir->isValid()) {
+            m_tempDirLock = std::make_unique<QLockFile>(m_tempDir->filePath(u"lockfile"_s));
+            m_tempDirLock->setStaleLockTime(0);
+            m_tempDirLock->tryLock();
+        }
+        auto future = QtConcurrent::run(removeUnlockedDirs, oldDirs);
     }
     return m_tempDir->isValid() ? m_tempDir.get() : nullptr;
 }
@@ -205,6 +220,42 @@ QString ExportManager::truncatedFilename(QString const &filename) const
     return result;
 }
 
+inline static void removeEmptyPlaceholderAndSeparators(const QString &placeholder, QString &string)
+{
+    using QRE = QRegularExpression;
+    // Exclude word characters from being counted as separators.
+    // NOTE: The parentheses are for a raw string literal, not a regex capture group.
+    static constexpr auto wordSymbol = uR"(\p{L}\p{M}\p{N})";
+    // Match 1 or more separator characters.
+    // Exclude dir separators because we don't want to accidentally mess up folder layouts.
+    // Exclude potential ends of tokens to protect tokens before the placeholder.
+    static const auto separatorsBefore = u"[^%1/>]+"_s.arg(wordSymbol);
+    // Exclude potential beginnings of tokens to protect tokens after the placeholder.
+    static const auto separatorsAfter = u"[^%1/<]+"_s.arg(wordSymbol);
+
+    // You can use the same placeholder multiple times with different combinations of dirs,
+    // separator chars, other characters or just text. Because of that, we need to match all of
+    // these. I don't want to make a massive regex because that would be a PITA to read and debug.
+
+    const auto beforeAndAfter = u"%1?%2%3?"_s.arg(separatorsBefore, placeholder, separatorsAfter);
+    const QRE wholeString(u"^%1$"_s.arg(beforeAndAfter));
+    if (string.contains(wholeString)) {
+        // Replace placeholder with default screenshot name.
+        string.replace(placeholder, u"Screenshot"_s);
+        return; // We parsed the whole string already.
+    }
+    const QRE betweenDirs(u"/%1/"_s.arg(beforeAndAfter));
+    string.replace(betweenDirs, u"/"_s);
+    // Remove these first so that text before the placeholder is more likely to be preserved.
+    const QRE after(placeholder % separatorsAfter);
+    string.remove(after);
+    // 
+    const QRE before(separatorsBefore % placeholder);
+    string.remove(before);
+    // Just remove the placeholder if none of the other matches work.
+    string.remove(placeholder);
+}
+
 QString ExportManager::formattedFilename(const QString &nameTemplate) const
 {
     const QDateTime timestamp = m_timestamp;
@@ -216,19 +267,13 @@ QString ExportManager::formattedFilename(const QString &nameTemplate) const
         title.replace(u'/', u'_'); // POSIX doesn't allow "/" in filenames
         result.replace("<title>"_L1, title);
     } else {
-        // We exclude word characters from being counted as separators.
-        static const auto wordSymbol = uR"(\p{L}\p{M}\p{N})"_s;
-        // Match 1 or more separator characters followed by <title>
-        // We exclude potential ends of tokens to protect tokens before <title>.
-        static const auto match1 = u"[^%1>]+<title>"_s.arg(wordSymbol);
-        // Match <title> followed by 0 or more separator characters
-        // We exclude potential beginnings of tokens to protect tokens after <title>.
-        static const auto match2 = u"<title>[^%1<]*"_s.arg(wordSymbol);
-        // If matching %1, then match %1, else match %2
-        static const QRegularExpression re(u"(?(?=%1)%1|%2)"_s.arg(match1, match2));
-        result.remove(re);
+        removeEmptyPlaceholderAndSeparators(u"<title>"_s, result);
     }
 
+    // Remove duplicate dir separators.
+    // These could come from user mistakes or empty placeholders being removed.
+    result.replace(QRegularExpression(u"/+"_s), u"/"_s);
+
     const auto &locale = QLocale::system();
     // QDateTime
     for (auto it = filenamePlaceholders.cbegin(); it != filenamePlaceholders.cend(); ++it) {
