@@ -153,7 +153,8 @@ SpectacleCore::SpectacleCore(QObject *parent)
             bool includePointer = m_cliOptions[CommandLineOptions::Pointer];
             includePointer |= m_startMode != StartMode::Background && Settings::videoIncludePointer();
             const auto &output = m_outputUrl.isLocalFile() ? videoOutputUrl() : QUrl();
-            m_videoPlatform->startRecording(output, VideoPlatform::Region, rect, includePointer);
+            static const auto rectKey = u"rect"_s;
+            m_videoPlatform->startRecording(output, VideoPlatform::Region, {{rectKey, rect}}, includePointer);
         } else {
             deleteWindows();
             m_annotationDocument->cropCanvas(rect);
@@ -186,14 +187,20 @@ SpectacleCore::SpectacleCore(QObject *parent)
         SpectacleWindow::setTitleForAll(SpectacleWindow::Unsaved);
         SpectacleWindow::setVisibilityForAll(QWindow::FullScreen);
     });
-    connect(imagePlatform, &ImagePlatform::newScreenshotFailed, this, [this](const QString message) {
-        auto uiMessage = i18nc("@info", "An error occurred while taking a screenshot.");
+
+    // The behavior happens to be very similar right now, so we use a shared base function
+    auto onScreenshotOrRecordingFailed = [this](const QString &message,
+                                                QString uiMessage,
+                                                void (SpectacleCore::*dBusFailedSignal)(const QString &),
+                                                void (ViewerWindow::*showViewerMessage)(const QString &)) {
         if (!message.isEmpty()) {
             uiMessage = uiMessage % u"\n"_s % message;
         }
         const auto &windows = qGuiApp->allWindows();
         const bool hasVisibleWindow = std::any_of(windows.cbegin(), windows.cend(), [](auto *window){
-            return window->isVisible();
+            // Is visible and is a normal window or dialog (no tooltips or menus)
+            const auto typeFlags = window->flags() & Qt::WindowType_Mask;
+            return window->isVisible() && (typeFlags == Qt::Window || typeFlags == Qt::Dialog);
         });
         switch (m_startMode) {
         case StartMode::Background:
@@ -204,7 +211,7 @@ SpectacleCore::SpectacleCore(QObject *parent)
             return;
         case StartMode::DBus: {
             showErrorMessage(uiMessage);
-            Q_EMIT dbusScreenshotFailed(message);
+            Q_EMIT (this->*dBusFailedSignal)(message);
             if (!hasVisibleWindow) {
                 Q_EMIT allDone();
             }
@@ -220,10 +227,56 @@ SpectacleCore::SpectacleCore(QObject *parent)
             qWarning().noquote() << message;
             auto viewer = ViewerWindow::instance();
             viewer->setVisible(true);
-            viewer->showScreenshotFailedMessage(uiMessage);
+            (viewer->*showViewerMessage)(uiMessage);
             return;
         }
         }
+    };
+    connect(imagePlatform, &ImagePlatform::newScreenshotFailed, this, [onScreenshotOrRecordingFailed](const QString message) {
+        auto uiMessage = i18nc("@info", "An error occurred while taking a screenshot.");
+        onScreenshotOrRecordingFailed(message, uiMessage, &SpectacleCore::dbusScreenshotFailed, &ViewerWindow::showScreenshotFailedMessage);
+    });
+
+    s_systemTrayIcon = std::make_unique<QSystemTrayIcon>(QIcon::fromTheme(u"media-record-symbolic"_s));
+    auto systemTrayIcon = s_systemTrayIcon.get();
+    connect(systemTrayIcon, &QSystemTrayIcon::activated, systemTrayIcon, [](auto reason) {
+        if (reason == QSystemTrayIcon::Trigger) {
+            SpectacleCore::instance()->finishRecording();
+        }
+    });
+
+    auto videoPlatform = m_videoPlatform.get();
+    connect(videoPlatform, &VideoPlatform::recordingChanged,
+            systemTrayIcon, [this](bool isRecording){
+        s_systemTrayIcon->setVisible(isRecording);
+        if (!isRecording) {
+            m_captureWindows.clear();
+        }
+    });
+    connect(videoPlatform, &VideoPlatform::recordedTimeChanged, this, [this] {
+        Q_EMIT recordedTimeChanged();
+        s_systemTrayIcon->setToolTip(i18nc("@info:tooltip", "Spectacle is recording: %1\nClick to finish recording", recordedTime()));
+    });
+    connect(videoPlatform, &VideoPlatform::recordingSaved, this, [this](const QUrl &fileUrl) {
+        // Always try to save. Needed to move recordings out of temp dir.
+        ExportManager::instance()->exportVideo(autoExportActions() | ExportManager::Save, fileUrl, videoOutputUrl());
+    });
+    connect(videoPlatform, &VideoPlatform::recordingCanceled, this, [this] {
+        if (m_startMode != StartMode::Gui || isGuiNull()) {
+            Q_EMIT allDone();
+            return;
+        }
+        SpectacleWindow::setTitleForAll(SpectacleWindow::Previous);
+    });
+    connect(videoPlatform, &VideoPlatform::recordingFailed, this, [onScreenshotOrRecordingFailed](const QString &message){
+        auto uiMessage = i18nc("@info", "An error occurred while attempting to record the screen.");
+        onScreenshotOrRecordingFailed(message, uiMessage, &SpectacleCore::dbusRecordingFailed, &ViewerWindow::showRecordingFailedMessage);
+    });
+    connect(videoPlatform, &VideoPlatform::regionRequested, this, [this] {
+        SelectionEditor::instance()->reset();
+        initCaptureWindows(CaptureWindow::Video);
+        SpectacleWindow::setTitleForAll(SpectacleWindow::Unsaved);
+        SpectacleWindow::setVisibilityForAll(QWindow::FullScreen);
     });
 
     // set up the export manager
@@ -354,6 +407,12 @@ SpectacleCore::SpectacleCore(QObject *parent)
     connect(imagePlatform, &ImagePlatform::supportedGrabModesChanged, captureModeModel, [this](){
         m_captureModeModel->setGrabModes(m_imagePlatform->supportedGrabModes());
     });
+    connect(videoPlatform, &VideoPlatform::supportedRecordingModesChanged, m_recordingModeModel.get(), [this](){
+        m_recordingModeModel->setRecordingModes(m_videoPlatform->supportedRecordingModes());
+    });
+    connect(videoPlatform, &VideoPlatform::supportedFormatsChanged, m_videoFormatModel.get(), [this](){
+        m_videoFormatModel->setFormats(m_videoPlatform->supportedFormats());
+    });
 
     connect(qApp, &QApplication::screenRemoved, this, [this](QScreen *screen) {
         // It's dangerous to erase from within a for loop, so we use std::find_if
@@ -365,58 +424,6 @@ SpectacleCore::SpectacleCore(QObject *parent)
             m_captureWindows.erase(it);
         }
     });
-
-    s_systemTrayIcon = std::make_unique<QSystemTrayIcon>(QIcon::fromTheme(u"media-record-symbolic"_s));
-    auto systemTrayIcon = s_systemTrayIcon.get();
-    connect(systemTrayIcon, &QSystemTrayIcon::activated, systemTrayIcon, [](auto reason) {
-        if (reason == QSystemTrayIcon::Trigger) {
-            SpectacleCore::instance()->finishRecording();
-        }
-    });
-
-    auto videoPlatform = m_videoPlatform.get();
-    connect(videoPlatform, &VideoPlatform::recordingChanged,
-            systemTrayIcon, [this](bool isRecording){
-        s_systemTrayIcon->setVisible(isRecording);
-        if (!isRecording) {
-            m_captureWindows.clear();
-        }
-    });
-    connect(videoPlatform, &VideoPlatform::recordedTimeChanged, this, [this] {
-        Q_EMIT recordedTimeChanged();
-        s_systemTrayIcon->setToolTip(i18nc("@info:tooltip", "Spectacle is recording: %1\nClick to finish recording", recordedTime()));
-    });
-    connect(videoPlatform, &VideoPlatform::recordingSaved, this, [this](const QUrl &fileUrl) {
-        // Always try to save. Needed to move recordings out of temp dir.
-        ExportManager::instance()->exportVideo(autoExportActions() | ExportManager::Save, fileUrl, videoOutputUrl());
-    });
-    connect(videoPlatform, &VideoPlatform::recordingCanceled, this, [this] {
-        if (m_startMode != StartMode::Gui || isGuiNull()) {
-            Q_EMIT allDone();
-            return;
-        }
-        SpectacleWindow::setTitleForAll(SpectacleWindow::Previous);
-    });
-    connect(videoPlatform, &VideoPlatform::recordingFailed, this, [this](const QString &message){
-        switch (m_startMode) {
-        case StartMode::Background:
-            if (!message.isEmpty()) {
-                showErrorMessage(message);
-            }
-            Q_EMIT allDone();
-            return;
-        case StartMode::DBus:
-            Q_EMIT dbusRecordingFailed(message);
-            Q_EMIT allDone();
-            return;
-        case StartMode::Gui:
-            if (!ViewerWindow::instance()) {
-                initViewerWindow(ViewerWindow::Dialog);
-            }
-            ViewerWindow::instance()->showRecordingFailedMessage(message);
-            return;
-        }
-    });
 }
 
 SpectacleCore::~SpectacleCore() noexcept
@@ -1158,15 +1165,8 @@ void SpectacleCore::startRecording(VideoPlatform::RecordingMode mode, bool withP
     }
     m_lastRecordingMode = mode;
     setVideoMode(true);
-    if (mode == VideoPlatform::Region) {
-        SelectionEditor::instance()->reset();
-        initCaptureWindows(CaptureWindow::Video);
-        SpectacleWindow::setTitleForAll(SpectacleWindow::Unsaved);
-        SpectacleWindow::setVisibilityForAll(QWindow::FullScreen);
-    } else {
-        const auto &output = m_outputUrl.isLocalFile() ? videoOutputUrl() : QUrl();
-        m_videoPlatform->startRecording(output, mode, {}, withPointer);
-    }
+    const auto &output = m_outputUrl.isLocalFile() ? videoOutputUrl() : QUrl();
+    m_videoPlatform->startRecording(output, mode, {}, withPointer);
 }
 
 void SpectacleCore::finishRecording()
